package com.ruoyi.common.utils.file;

import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.utils.StringUtils;
import org.apache.poi.util.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.Arrays;

/**
 * 图片处理工具类
 *
 * @author ruoyi
 */
public class ImageUtils {
    private static final Logger log = LoggerFactory.getLogger(ImageUtils.class);

    public static byte[] getImage(String imagePath) {
        InputStream is = getFile(imagePath);
        try {
            return IOUtils.toByteArray(is);
        } catch (Exception e) {
            log.error("图片加载异常 {}", e);
            return null;
        } finally {
            IOUtils.closeQuietly(is);
        }
    }

    public static InputStream getFile(String imagePath) {
        try {
            byte[] result = readFile(imagePath);
            result = Arrays.copyOf(result, result.length);
            return new ByteArrayInputStream(result);
        } catch (Exception e) {
            log.error("获取图片异常 {}", e);
        }
        return null;
    }

    /**
     * 读取文件为字节数据
     *
     * @param url 地址
     * @return 字节数据
     */
    public static byte[] readFile(String url) {
        InputStream in = null;
        try {
            if (url.startsWith("http")) {
                // 网络地址
                URL urlObj = new URL(url);
                URLConnection urlConnection = urlObj.openConnection();
                urlConnection.setConnectTimeout(30 * 1000);
                urlConnection.setReadTimeout(60 * 1000);
                urlConnection.setDoInput(true);
                in = urlConnection.getInputStream();
            } else {
                // 本机地址
                String localPath = RuoYiConfig.getProfile();
                String downloadPath = localPath + StringUtils.substringAfter(url, Constants.RESOURCE_PREFIX);
                in = new FileInputStream(downloadPath);
            }
            return IOUtils.toByteArray(in);
        } catch (Exception e) {
            log.error("获取文件路径异常 {}", e);
            return null;
        } finally {
            IOUtils.closeQuietly(in);
        }
    }


    /**
     * 添加图片水印
     *
     * @param sourceFile    文件源路径
     * @param destFile      文件路径
     * @param watermarkText1 水印信息上
     * @param watermarkText2 水印信息下
     * @throws Exception 异常
     */
    public static void addWatermark(File sourceFile, File destFile, String watermarkText1, String watermarkText2) throws Exception {
        // 读取图片
        BufferedImage image = ImageIO.read(sourceFile);

        // 获取图片宽高
        int width = image.getWidth();
        int height = image.getHeight();

        // 创建 Graphics2D 对象
        Graphics2D g2d = (Graphics2D) image.getGraphics();
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        // 根据图片大小调整字体大小
        int fontSize = Math.max(width, height) / 40; // 字体大小为图片较大边长的 1/20
        g2d.setFont(new Font("SimHei", Font.BOLD, fontSize));
        g2d.setColor(new Color(255, 255, 255, 128)); // 白色半透明

        // 计算水印文本宽高
        FontMetrics fontMetrics = g2d.getFontMetrics();
        // 第一行水印的宽高
        int textWidth1 = fontMetrics.stringWidth(watermarkText1);
        int textHeight = fontMetrics.getHeight(); // 每行的高度

        // 第二行水印的宽度
        int textWidth2 = fontMetrics.stringWidth(watermarkText2);

        // 计算水印的位置（右下角，分两行）
        int padding = width / 50; // 边距
        int x1 = width - textWidth1 - padding; // 第一行水平位置
        int x2 = width - textWidth2 - padding; // 第二行水平位置
        int y1 = height - textHeight - padding; // 第一行垂直位置
        int y2 = height - padding; // 第二行垂直位置

        // 绘制两行水印
        g2d.drawString(watermarkText1, x1, y1);
        g2d.drawString(watermarkText2, x2, y2);
        g2d.dispose();

        // 保存处理后的图片
        ImageIO.write(image, "jpg", destFile);
    }
}
